@@ -262,3 +262,31 @@ asymmetric.preference.game <- function(nodes, types,
         as.logical(loops),
         PACKAGE="igraph")
 }
+
+connect.neighborhood <- function(graph, order, mode="all") {
+  if (!is.igraph(graph)) {
+    stop("Not a graph object")
+  }
+  if (is.character(mode)) {
+    mode <- switch(mode, "out"=1, "in"=2, "all"=3, "total"=3)
+  }
+
+  .Call("R_igraph_connect_neighborhood", graph, as.numeric(order),
+        as.numeric(mode),
+        PACKAGE="igraph")
+}
+
+rewire.edges <- function(graph, prob) {
+  if (!is.igraph(graph)) {
+    stop("Not a graph object")
+  }
+  .Call("R_igraph_rewire_edges", graph, as.numeric(prob),
+        PACKAGE="igraph")
+}
+
+watts.strogatz.game <- function(dim, size, nei, p) {
+  .Call("R_igraph_watts_strogatz_game", as.numeric(dim), as.numeric(size),
+        as.numeric(nei), as.numeric(p),
+        PACKAGE="igraph")
+}
+
