@@ -98,7 +98,7 @@ int igraph_vs_all(igraph_vs_t *vs) {
  * Time complexity: O(1).
  */
 
-igraph_vs_t igraph_vss_all() {
+igraph_vs_t igraph_vss_all(void) {
   igraph_vs_t allvs;
   allvs.type=IGRAPH_VS_ALL;
   return allvs;  
@@ -194,7 +194,7 @@ int igraph_vs_none(igraph_vs_t *vs) {
  * Time complexity: O(1).
  */
 
-igraph_vs_t igraph_vss_none() {
+igraph_vs_t igraph_vss_none(void) {
   igraph_vs_t nonevs;
   nonevs.type=IGRAPH_VS_NONE;
   return nonevs;
@@ -338,6 +338,36 @@ int igraph_vs_vector_small(igraph_vs_t *vs, ...) {
   return 0;  
 }
 
+/**
+ * \function igraph_vs_vector_copy
+ * 
+ * This function makes it possible to handle a \type vector_t
+ * permanently as a vertex selector. The vertex selector creates a
+ * copy of the original vector, so the vector can safely be destroyed
+ * after creating the vertex selector. Changing the original vector
+ * will not affect the vertex selector. The vertex selector is
+ * responsible for deleting the copy made by itself.
+ * 
+ * \param vs Pointer to an uninitialized vertex selector.
+ * \param v Pointer to a \type igraph_vector_t object.
+ * \return Error code.
+ * 
+ * Time complexity: O(1).
+ */
+
+int igraph_vs_vector_copy(igraph_vs_t *vs,
+			  const igraph_vector_t *v) {
+  vs->type=IGRAPH_VS_VECTOR;
+  vs->data.vecptr=Calloc(1, igraph_vector_t);
+  if (vs->data.vecptr==0) {
+    IGRAPH_ERROR("Cannot create vertex selector", IGRAPH_ENOMEM);
+  }
+  IGRAPH_FINALLY(igraph_free, (igraph_vector_t*)vs->data.vecptr);
+  IGRAPH_CHECK(igraph_vector_copy((igraph_vector_t*)vs->data.vecptr, v));
+  IGRAPH_FINALLY_CLEAN(1);
+  return 0;
+}
+
 /**
  * \function igraph_vs_seq
  * 
@@ -760,7 +790,7 @@ int igraph_es_none(igraph_es_t *es) {
  * Time complexity: O(1).
  */
 
-igraph_es_t igraph_ess_none() {
+igraph_es_t igraph_ess_none(void) {
   igraph_es_t es;
   es.type=IGRAPH_ES_NONE;
   return es;
@@ -870,6 +900,7 @@ igraph_es_t igraph_ess_vector(const igraph_vector_t *v) {
 int igraph_es_fromto(igraph_es_t *es,
 		     igraph_vs_t from, igraph_vs_t to) {
   /* TODO */
+  return 0;
 }
 
 /**
@@ -1200,7 +1231,6 @@ int igraph_i_eit_multipairs(const igraph_t *graph,
 			    igraph_es_t es, igraph_eit_t *eit) {
   long int n=igraph_vector_size(es.data.path.ptr);
   long int no_of_nodes=igraph_vcount(graph);
-  long int i;
   
   if (n % 2 != 0) {
     IGRAPH_ERROR("Cannot create edge iterator from odd number of vertices",
