@@ -24,6 +24,18 @@
 #ifndef REST_RANDOM_H
 #define REST_RANDOM_H
 
+#undef __BEGIN_DECLS
+#undef __END_DECLS
+#ifdef __cplusplus
+# define __BEGIN_DECLS extern "C" {
+# define __END_DECLS }
+#else
+# define __BEGIN_DECLS /* empty */
+# define __END_DECLS /* empty */
+#endif
+
+__BEGIN_DECLS
+
 #include "types.h"
 
 #include <stdlib.h>
@@ -62,4 +74,6 @@ extern int igraph_rng_inited;
 
 #endif
 
+__END_DECLS
+
 #endif
